@@ -5,7 +5,7 @@ use warnings;
 
 use Carp ();
 
-our $VERSION = '0.11';
+our $VERSION = '0.12';
 our $PREFIX  = '__cag_';
 
 require XSLoader;
@@ -26,18 +26,14 @@ sub import {
     my %opts = ref($_[0]) eq 'HASH' ? %{ $_[0] } : @_;
 
     for my $type (keys %opts) {
-        my $type_info = $REGISTERED_TYPES->{$type};
-
-        if (!defined $type_info) {
-            Carp::confess("Don't know how to install '$type' accessors");
-        }
-
         my $accessors = $opts{$type};
+        my $installer = _type_installer($type);
+
         if (ref($accessors) eq 'HASH') {
-            $type_info->{installer}->($class, $_, $accessors->{$_}) for keys %$accessors;
+            $installer->($class, $_, $accessors->{$_}) for keys %$accessors;
 
         } elsif (ref($accessors) eq 'ARRAY') {
-            $type_info->{installer}->($class, $_, $_) for @$accessors;
+            $installer->($class, $_, $_) for @$accessors;
 
         } else {
             Carp::confess("Can't understand format for '$type' accessors initializer");
@@ -47,36 +43,38 @@ sub import {
 
 sub mk_inherited_accessors {
     my $class = shift;
-
-    for my $entry (@_) {
-        if (ref($entry) eq 'ARRAY') {
-            _mk_inherited_accessor($class, @$entry);
-        } else {
-            _mk_inherited_accessor($class, $entry, $entry);
-        }
-    }
+    mk_type_accessors($class, 'inherited', @_);
 }
 
 sub mk_class_accessors {
     my $class = shift;
-
-    _mk_class_accessor($class, $_) for (@_);
+    mk_type_accessors($class, 'class', @_);
 }
 
 sub mk_varclass_accessors {
     my $class = shift;
+    mk_type_accessors($class, 'varclass', @_);
+}
 
-    _mk_varclass_accessor($class, $_) for (@_);
+sub mk_type_accessors {
+    my ($class, $type) = (shift, shift);
+
+    my $installer = _type_installer($type);
+    for my $entry (@_) {
+        if (ref($entry) eq 'ARRAY') {
+            $installer->($class, @$entry);
+        } else {
+            $installer->($class, $entry, $entry);
+        }
+    }
 }
 
 sub register_types {
-    my %args = @_;
-
-    while (my ($key, $val) = each %args) {
-        register_type($key, $val);
-    }
+    register_type(shift, shift) while scalar @_;
 }
 
+sub is_type_registered { exists $REGISTERED_TYPES->{$_[0]} }
+
 sub register_type {
     my ($type, $args) = @_;
 
@@ -98,6 +96,13 @@ sub register_type {
 #   Functions below are NOT part of the public API
 #
 
+sub _type_installer {
+    my $type = shift;
+
+    my $type_info = $REGISTERED_TYPES->{$type} or Carp::confess("Don't know how to install '$type' accessors");
+    return $type_info->{installer};
+}
+
 sub _mk_inherited_accessor {
     my ($class, $name, $field) = @_;
 
@@ -155,19 +160,20 @@ by the L<Class::Accessor::Grouped> module. They give you a capability to overrid
 a parent class with values set in childs or object instances. Generated accessors are compatible with
 L<Class::Accessor::Grouped> generated ones.
 
-Since this module focuses primary on speed, it provides no capability to have your own per-class
+Since this module focuses primary on speed, it provides no means to have your own per-class
 getters/setters logic (like overriding L<get_inherited>/L<set_inherited> in L<Class::Accessor::Grouped>),
-but it gives you an ability to register a single get/set callback for you own accessor types.
+but it allows you to register a single get/set callback per accessor type.
 
-It also provides two types of non-inherited accessors - 'class' and 'varclass' with the only difference
-that 'varclass' internal storage is an alias to a package variable of the same name. They always give you
-package value, even when called on an object.
+It also provides two types of non-inherited accessors, 'class' and 'varclass', which give you values
+from a package they were defined in, even when called on objects. The difference between them is that
+the 'varclass' internal storage is a package variable with the same name, while 'class' stores it's value
+in an anonymous variable.
 
 =head1 UTF-8
 
-Starting with the perl 5.16.0, this module provides full support for UTF-8 method names and hash 
-keys. Before that, you can't distinguish UTF-8 strings from bytes string in method names, only in 
-hash keys. You have been warned.
+Starting with the perl 5.16.0, this module provides full support for UTF-8 (and binary) method names and hash
+keys. But in previous perls you can't distinguish UTF-8 strings from bytes string in method names, so accessors
+with UTF-8 names can end up getting a wrong value. You have been warned.
 
 =head1 THREADS
 
@@ -181,19 +187,22 @@ L<Class::Accessor::Inherited::XS> is at least 10x times faster than L<Class::Acc
 on your usage pattern. Accessing data from a parent in a large inheritance chain is still the worst case,
 but even there L<Class::Accessor::Inherited::XS> beats L<Class::Accessor::Grouped> best-case.
 
+Accessors with just an empty sub callback are ~3x times slower then normal ones, so use them only when you definitely need them.
+
 Here are results from a benchmark run on perl 5.20.1 (see bench folder):
 
-                          Rate pkg_gparent_cag pkg_cag obj_cag pkg_set_cag pkg_gparent_caixs pkg_caix pkg_set_caix obj_cxa obj_caix obj_direct
-  pkg_gparent_cag     237444/s              --    -77%    -79%        -82%              -92%     -97%         -97%    -99%     -99%       -99%
-  pkg_cag            1013067/s            327%      --    -11%        -21%              -66%     -88%         -89%    -94%     -95%       -96%
-  obj_cag            1137400/s            379%     12%      --        -12%              -62%     -87%         -87%    -93%     -94%       -95%
-  pkg_set_cag        1286220/s            442%     27%     13%          --              -57%     -85%         -86%    -92%     -93%       -95%
-  pkg_gparent_caixs  2995865/s           1162%    196%    163%        133%                --     -65%         -66%    -82%     -84%       -88%
-  pkg_caix           8602691/s           3523%    749%    656%        569%              187%       --          -3%    -49%     -54%       -65%
-  pkg_set_caix       8907800/s           3652%    779%    683%        593%              197%       4%           --    -47%     -52%       -64%
-  obj_cxa           16781840/s           6968%   1557%   1375%       1205%              460%      95%          88%      --     -10%       -32%
-  obj_caix          18642286/s           7751%   1740%   1539%       1349%              522%     117%         109%     11%       --       -25%
-  obj_direct        24807382/s          10348%   2349%   2081%       1829%              728%     188%         178%     48%      33%         --
+                        Rate pkg_gparent_cag pkg_cag obj_cag pkg_set_cag pkg_gparent_caixs obj_caix_cb pkg_caix pkg_set_caix obj_cxa obj_caix obj_direct
+pkg_gparent_cag     228862/s              --    -76%    -80%        -82%              -92%        -96%     -97%         -98%    -99%     -99%       -99%
+pkg_cag             942636/s            312%      --    -19%        -27%              -68%        -85%     -88%         -90%    -94%     -95%       -97%
+obj_cag            1158463/s            406%     23%      --        -10%              -61%        -81%     -86%         -87%    -93%     -94%       -96%
+pkg_set_cag        1287939/s            463%     37%     11%          --              -56%        -79%     -84%         -86%    -92%     -93%       -95%
+pkg_gparent_caixs  2958598/s           1193%    214%    155%        130%                --        -52%     -64%         -68%    -82%     -84%       -89%
+obj_caix_cb        6138858/s           2582%    551%    430%        377%              107%          --     -25%         -33%    -63%     -67%       -77%
+pkg_caix           8159797/s           3465%    766%    604%        534%              176%         33%       --         -11%    -51%     -56%       -70%
+pkg_set_caix       9162566/s           3904%    872%    691%        611%              210%         49%      12%           --    -45%     -51%       -66%
+obj_cxa           16699964/s           7197%   1672%   1342%       1197%              464%        172%     105%          82%      --     -10%       -39%
+obj_caix          18616018/s           8034%   1875%   1507%       1345%              529%        203%     128%         103%     11%       --       -32%
+obj_direct        27185300/s          11778%   2784%   2247%       2011%              819%        343%     233%         197%     63%      46%         --
 
 =head1 EXTENDING
 
@@ -210,16 +219,18 @@ Here are results from a benchmark run on perl 5.20.1 (see bench folder):
         inherited_cb => ['bar'],
     };
 
-You can register new inherited accessor types with associated read/write callbacks. They're still
-'inherited', but you can modify return values. Those new types can be installed using
-the L<Class::Accessor::Inherited::XS> import() call. Unlike L<Class::Accessor::Grouped>,
-here's a single callback per accessor type, without any inheritance lookups for get_*/set_* functions.
+    #or in a Class::Accessor::Grouped-like fashion
+    __PACKAGE__->mk_type_accessors(inherited_cb => 'foo', 'bar');
 
-B<on_read> callback gets a single argument - from a normal 'inherited' accessor. It's return value is a new
+You can register new inherited accessor types with associated read/write callbacks. Unlike
+L<Class::Accessor::Grouped>, only a single callback is set for a type, without per-class
+ get_*/set_* lookups.
+
+B<on_read> callback gets a single argument - from a normal 'inherited' accessor. It's return value is the new
 accessor's return value (and is not stored anywhere).
 
 B<on_write> callback gets two arguments - original args from the accessor's call. It's return value is saved
-instead of the user's supplied one. Exceptions thrown from this callback will cancel store and leave old value unchanged.
+instead of the user's supplied one. Exceptions thrown from this callback will cancel store and leave the old value unchanged.
 
 =head1 PROFILING WITH Devel::NYTProf
 
@@ -228,11 +239,11 @@ on the opcode level. To squeeze last bits of performance, L<Class::Accessor::Inh
 on each call site of its accessors. It turns out into CAIX favor - L<Devel::NYTProf> sees only first call to CAIX
 accessor, but all subsequent ones become invisible to the subs profiler.
 
-Note that statement profiler still correctly accounts time spent on a line, you just don't see time spent in accessors'
-calls separately. That's sometimes OK, sometimes not - you get profile with all possible optimizations on, but it's hard for understanding.
+Note that the statement profiler still correctly accounts for the time spent on each line, you just don't see time spent in accessors'
+calls separately. That's sometimes OK, sometimes not - you get profile with all possible optimizations on, but it's not easy to comprehend.
 
 Since it's hard to detect L<Devel::NYTProf> (and any other module doing such magic) in a portable way (all hail Win32), there's
-an %ENV switch available - you can set CAIXS_DISABLE_ENTERSUB to a true value to disable call site optimization and get full subs profile.
+an %ENV switch available - you can set CAIXS_DISABLE_ENTERSUB to a true value to disable opcode optimization and get a full subs profile.
 
 =head1 SEE ALSO
 
